﻿//
// Authors:
// Xiaoliang Pang (mailto:mv@live.cn)
//
// Copyright (c) 2010 Landpy Software
//
// http://www.cnblogs.com/pangxiaoliang
//
using System;
using System.Collections.Generic;
using System.Text;
using Landpy.AOP;
using Landpy.ProxyPattern;

namespace ArchitectTestForAOP
{
    /// <summary>
    /// The application.
    /// </summary>
    class Program
    {
        /// <summary>
        /// The start point of the application.
        /// </summary>
        /// <param name="args">The params.</param>
        static void Main(string[] args)
        {
            MyComputer myComputer = new MyComputer();
            ISite site = new AVSite();
            myComputer.Visit(site); // Visit AV site will be return warning.

            WriteSplitLine();

            site = new MyProxySite("http://www.AVSite.com"); // Use online web proxy.
            myComputer.Visit(site);

            WriteSplitLine();

            site = new CnblogsSite();
            myComputer.Visit(site);

            WriteSplitLine();

            // Proxy class to implement the AOP.
            IInterception interception = new MyInterception();
            AOPTestWithProxyClass aopTestTwo = ProxyFactory.CreateProxyInstance<AOPTestWithProxyClass>(interception) as AOPTestWithProxyClass;
            aopTestTwo.Show();

            WriteSplitLine();

            // Attribute to implement the AOP.
            AOPTestWithAttribute aopTest = new AOPTestWithAttribute();
            aopTest.Show();

            WriteSplitLine();

            AOPTestWithAttributForTrasaction aopTestWithAttributForTrasaction = new AOPTestWithAttributForTrasaction();
            // Execute the transaction successfully.
            aopTestWithAttributForTrasaction.ExecuteTransactionSuccessfully();

            WriteSplitLine();

            // Execute the transaction unsuccessfully.
            try
            {   
                aopTestWithAttributForTrasaction.ExecuteTransactionUnsuccessfully();
            }
            catch (Exception ex)
            {
                Console.ForegroundColor = ConsoleColor.Blue;
                Console.WriteLine(ex.Message);
                Console.ForegroundColor = ConsoleColor.Gray;
            }

            Console.WriteLine("Press any key to continue.");
            Console.ReadKey();
        }

        private static void WriteSplitLine()
        {
            #region Split line

            Console.ForegroundColor = ConsoleColor.Yellow;
            Console.WriteLine("-------------------------Split line in the story.---------------------");
            Console.ForegroundColor = ConsoleColor.White;

            #endregion
        }
    }

    /// <summary>
    /// The interception of the AOP.
    /// </summary>
    class MyInterception : IInterception
    {

        #region IInterception Members

        /// <summary>
        /// Before invoke the real instance to do something.
        /// </summary>
        public void PreInvoke()
        {
            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("===Pre MyInterception.===");
            Console.ForegroundColor = ConsoleColor.Gray;
        }

        /// <summary>
        /// End invoke the real instance to do something.
        /// </summary>
        public void PostInvoke()
        {
            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("===Post MyInterception.===");
            Console.ForegroundColor = ConsoleColor.Gray;
        }

        /// <summary>
        /// Handling the exception which occurs when the method is invoked.
        /// </summary>
        public void ExceptionHandle()
        {
            Console.ForegroundColor = ConsoleColor.Red;
            Console.WriteLine("There is a exception!");
            Console.ForegroundColor = ConsoleColor.Gray;
        }

        #endregion
    }

    /// <summary>
    /// The interception of the AOP for trasaction.
    /// </summary>
    class TransactionInterception : IInterception
    {
        #region IInterception Members

        public void ExceptionHandle()
        {
            Console.ForegroundColor = ConsoleColor.Red;
            Console.WriteLine("☆☆☆Rollback transaction☆☆☆");
            Console.ForegroundColor = ConsoleColor.Gray;
        }

        public void PostInvoke()
        {
            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("☆☆☆Commit transaction☆☆☆");
            Console.ForegroundColor = ConsoleColor.Gray;
        }

        public void PreInvoke()
        {
            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("☆☆☆Begin transaction☆☆☆");
            Console.ForegroundColor = ConsoleColor.Gray;
        }

        #endregion
    }

    /// <summary>
    /// The test class for AOP use attribute..
    /// </summary>
    [AOPProxy(Interception = typeof(MyInterception))]
    class AOPTestWithAttribute : ContextBoundObject
    {
        public void Show()
        {
            Console.ForegroundColor = ConsoleColor.Gray;
            Console.WriteLine("Hello, I am AOPTest.");
            Console.ForegroundColor = ConsoleColor.Gray;
        }
    }

    /// <summary>
    /// The test class for AOP use proxy class.
    /// </summary>
    class AOPTestWithProxyClass : ContextBoundObject
    {
        public void Show()
        {
            Console.ForegroundColor = ConsoleColor.Gray;
            Console.WriteLine("Hello, I am AOPTestTwo.");
            Console.ForegroundColor = ConsoleColor.Gray;
        }
    }

    [AOPProxy(Interception = typeof(TransactionInterception))]
    class AOPTestWithAttributForTrasaction : ContextBoundObject
    {
        public void ExecuteTransactionSuccessfully()
        {
            Console.WriteLine("Execute the transaction successfully:)");
        }

        public void ExecuteTransactionUnsuccessfully()
        {
            Console.WriteLine("Execute the transaction unsuccessfully:(");
            throw new AOPNullException();
        }
    }
}
